@@ -1,5 +1,64 @@
 from bs4 import BeautifulSoup
-import lxml, os, copy, pandas, time, datetime, requests
+import lxml, os, copy, pandas, time, datetime, requests, sys
+
+class SSParse:
+    
+    def help():
+        
+        print("SSParse:")
+        
+        print("\t[1] __init__(self, filename)")
+        
+        print("\t[2] fetch(self, key) # folder , name , extension \n")
+    
+    def __init__(self, filename):
+        
+        if (type(filename) != str):
+            
+            print(f"Warning [SSParse]: {filename} is not of type string.\n")
+            
+            self.valid = False
+            
+        else:
+            
+            self.valid = True
+            
+            self.filename = filename
+            
+            self.__parse_name()
+            
+    def __parse_name(self):
+        
+        _extension = (f".{self.filename.split('.')[-1]}" if ("." in self.filename) else "")
+        
+        name = self.filename[:len(self.filename) - len(_extension)]
+        
+        _name = (name.split("/")[-1] if ("/" in name) else name)
+        
+        _folder = name[:len(name) - len(_name)]
+        
+        self.dictionary = {
+            "name" : _name,
+            "folder" : _folder,
+            "extension" : _extension
+        }
+        
+    def fetch(self, key):
+        
+        if (self.valid):
+            
+            if (key in self.dictionary):
+                
+                return self.dictionary[key]
+            
+            else:
+                
+                print(f"Warning [SSParse]: {key} is not a valid key for fetching.\n")
+                
+        else:
+            
+            print("Warning [SSParse]: invalid filename.\n")
+            
 class SSFormat:
     
     class __Dict:
@@ -206,16 +265,104 @@ class SSData:
         )
         
         self.__save_data()
+
+class SSSave:
     
+    def help():
+        
+        print("SSSave:")
+        
+        print("\t[1] __init__(self, filename, directlink, buffer = 4096, progress_bar = False, overwrite = None)")
+        
+        print("\t[2] download(self)\n")
+    
+    def __init__(self, filename, directlink, buffer = 4096, progress_bar = False, overwrite = None):
+        
+        self.buffer, self.progress_bar, self.overwrite = buffer, progress_bar, overwrite
+        
+        if ((type(filename) != str) or (type(directlink) != str)):
+            
+            print("Warning [SSSave]: arguments must be of string type.\n")
+            
+            self.valid = False
+            
+        else:
+            
+            self.filename, self.directlink, self.valid = self.__file_exists(filename), directlink, True
+            
+    def __file_exists(self, filename):
+        
+        if ((self.overwrite != True) and (os.path.isfile(filename))):
+            
+            fileParse = SSParse(filename)
+            
+            if (self.overwrite == None):
+                
+                while True:
+                    
+                    print(f"Warning [SSSave]: file {filename} already exists, overwrite? (Y/N)")
+                    
+                    userInput = input("> ").lower()
+                    
+                    if ("y" in userInput):
+                        
+                        return filename
+                    
+                    elif ("n" in userInput):
+                        
+                        break
+            
+            return datetime.datetime.now().strftime(f"{fileParse.fetch('folder')}ssfile_%y%m%d%H%M%S{fileParse.fetch('extension')}")
+        
+        else:
+            
+            return filename
+    
+    def __show_progress(self, current, total):
+        
+        if (self.progress_bar):
+        
+            sys.stdout.write(f"\rDownloading [{self.directlink}] -> [{self.filename}] ({current/total*100}%)")
+            
+            sys.stdout.flush()
+    
+    def download(self):
+        
+        if (self.valid):
+            
+            fileData = requests.get(self.directlink, stream = True)
+            
+            length = int(fileData.headers.get("content-length"))
+            
+            temp = 0
+            
+            self.__show_progress(temp, length)
+            
+            with open(self.filename, "wb") as writeFile:
+                
+                for data in fileData.iter_content(chunk_size = self.buffer):
+                    
+                    writeFile.write(data)
+                    
+                    self.__show_progress(temp, length)
+                    
+                    temp += len(data)
+                    
+            self.__show_progress(temp, length)
+        
+        else:
+            
+            print("Warning [SSSave]: cannot download file because of unsupported arguments.\n")
+        
 class StaticScraper:
     
     def help():
         
         print("StaticScraper(): ")
         
-        print("\t [1]__init__(self, info)\n")
+        print("\t[1]__init__(self, info)")
         
-        print("\t [2]scrape(self, start = 1, pages = 1)\n")
+        print("\t[2]scrape(self, start = 1, pages = 1)\n")
     
     def __init__(self, info):
         
@@ -335,4 +482,11 @@ info = SSInfo(website, pageformat, itemformat, attrList)
 data = SSData(info, timesleep = 2)
 scraper = StaticScraper(data)
 scraper.scrape(start = 1, pages = 5)
+"""
+"""
+SSSave.help()
+
+file = SSSave("temp.json", "link", progress_bar = True, overwrite = False)
+
+file.download()
 """
\ No newline at end of file
